local sql=[[

CREATE TABLE mailsettings(
tk          INTEGER PRIMARY KEY,
key         TEXT NOT NULL,
value       TEXT DEFAULT NULL
);
INSERT INTO mailsettings (key, value) values("version", "1.0");
INSERT INTO mailsettings (key, value) values("ticktime", "60000");
INSERT INTO mailsettings (key, value) values("domainname", "");
INSERT INTO mailsettings (key, value) values("confirmpwdsec", "1");

CREATE TABLE mailsmtp(
key           INTEGER PRIMARY KEY,
email         TEXT NOT NULL,
server        TEXT NOT NULL,
port          INTEGER NOT NULL,
useauth       INTEGER NOT NULL, 
uname         TEXT NOT NULL,
password      TEXT NOT NULL,
accounttype   TEXT NOT NULL,
consec        TEXT NOT NULL
);
INSERT INTO mailsmtp (email,server,port,useauth,uname,password,accounttype,consec) values('','',25,0,'','','generic','none');



-- Contact's master list.
CREATE TABLE mailcontact(
key          INTEGER PRIMARY KEY,
email        TEXT NOT NULL,
firstname    TEXT,
lastname     TEXT,
id           TEXT DEFAULT NULL, -- Optional opaque ID set by mail manager.
hash         TEXT NOT NULL, -- A 32 byte unique user key. The hash is used when creating download links.
password     TEXT DEFAULT NULL,
listkey      INTEGER REFERENCES maillist(key) NOT NULL, -- Can be in one list only.
mailcnt      INTEGER DEFAULT 0, -- How many times this contact has read messages. SA tracker:cnt.
mailtm       INTEGER DEFAULT 0, -- Last time this user accessed any mail. SA tracker:time.
time         INTEGER NOT NULL  -- When added.
);

-- Contacts can be in one list.
CREATE TABLE maillist(
key         INTEGER PRIMARY KEY,
name        TEXT NOT NULL,
description TEXT
);

-- Contacts are part of one or several email groups.
CREATE TABLE mailgroup(
key         INTEGER PRIMARY KEY,
name        TEXT NOT NULL,
description TEXT
);

-- A connector for contacts (sub) and the groups subscribed.
CREATE TABLE mailsubgroup(
mlkey    INTEGER REFERENCES mailcontact(key) NOT NULL,
listkey  INTEGER REFERENCES maillist(key) NOT NULL,
groupkey INTEGER REFERENCES mailgroup(key) NOT NULL
);


-- External programs run as post jobs prior to the download
CREATE TABLE mailbatchjob(
key      INTEGER PRIMARY KEY,
name     TEXT NOT NULL,
cmd      TEXT NOT NULL   -- The command to run on download
);



-- A job is created when a user send emails to (one or several lists and/or one or several groups) or individuals
-- A job is also created when sending password confirmation emails. All defaults are NULL, except listkey, which is the lists (JSON enc).
CREATE TABLE mailjob(
key      INTEGER PRIMARY KEY,
bduser   TEXT NOT NULL, -- The FuguHub user that sent the download link
fromaddr TEXT NOT NULL, -- The sender email address
-- hash: A random 2 byte user key. The hash is used when creating download links. The hash is null if not sending links
hash     TEXT DEFAULT NULL,
subject  TEXT NOT NULL, -- Email subject
txtbody  TEXT NOT NULL, -- Email text body
htmlbody TEXT NOT NULL, -- Email html body

riop     DEFAULT NULL,  -- Root IO Path

-- Note: tokey is NULL if sending to list or group, otherwise tokey is 'to', listkey is 'cc' and groupkey is 'bcc'
tokey    TEXT DEFAULT NULL, -- JSON encoded array of mailcontact keys -- Send to the contacts listed here
listkey  TEXT DEFAULT NULL, -- JSON encoded array of maillist keys -- Send to the contacts listed here
groupkey TEXT DEFAULT NULL, -- JSON encoded array of mailgroup keys -- Send to the groups listed here

active   INTEGER DEFAULT NULL, -- Boolean, 1: in progress, 0: job done
sendcnt  INTEGER DEFAULT NULL, -- The number of emails sent. Used if job is resumed

attach   TEXT DEFAULT NULL, -- JSON encoded array of file names if attaching to body of email, otherwise NULL
secure   INTEGER DEFAULT NULL, -- Boolean, 1: must use SSL, 0: Both HTTP and HTTPS ok
password INTEGER DEFAULT NULL, -- Boolean, 1: must use password, 0: No password required
write    INTEGER DEFAULT NULL, -- Boolean, 1: Write and WebDAV access, 0: Download only. Not implemented!!!
expires  INTEGER DEFAULT NULL, -- Time when the link expires. Never expires if 0
date     INTEGER NOT NULL,     --  Date created
batchkey INTEGER REFERENCES mailbatchjob(key) DEFAULT NULL
);


-- The files (download links) in mailjob
CREATE TABLE mailjobfile(
key         INTEGER PRIMARY KEY,
mailjobkey  INTEGER REFERENCES mailjob(key) NOT NULL,
name        TEXT NOT NULL -- File Name
);

-- Download History
CREATE TABLE maildownload(
key        INTEGER PRIMARY KEY,
mailjobkey INTEGER REFERENCES mailjob(key) NOT NULL,
filekey    INTEGER REFERENCES mailjobfile(key) NOT NULL,
mlkey      INTEGER REFERENCES mailcontact(key) NOT NULL,
ipaddr     TEXT NOT NULL,
date       INTEGER NOT NULL,
err        TEXT DEFAULT NULL
);

]]



local env, conn = bd.openDb("bd")
local ok,err,serr=conn:mexec(sql)
bd.closeDb(env, conn)

trace("-------------- Creating the FuguHub Database ",
      ok and "-----------" or (serr and string.format("%s: %s",err,serr) or err))

