
local fmt=string.format
local quote=luasql.quotestr
local sqlutil=require"sqlutil"
local mailutil=require"mailutil"
local rio=ba.openio"root"

local function redir2securl(_ENV)
   response:sendredirect(bd.getSecureUrl(request,nil,true))
end


local function expiredinfo(_ENV)
   title="Link Expired"
   response:include"/rtl/.EmptyHeader.lsp"
   response:write'<h1><br/>Link Expired!</h1>'
   response:include"/rtl/.EmptyFooter.lsp"
   return true
end


local checkpasswd -- func
do
   local time,key
   local function getKey() -- Create a key valid for 20 mins
      if not time or (time + 20*60) < os.time() then
         time=os.time()
         key=ba.aeskey(32)
      end
      return time
   end
   local function _checkpasswd(_ENV,password)
      if password then
         local password2=bd.trim(request:data"password")
         if password2 then
            if bd.aesdecode(password) == password2 then
               local cookie = response:createcookie"lfmpwd"
               cookie:value(ba.aesencode(getKey(),password2))
               cookie:activate()
               return true
            end
         end
         local cookie = request:cookie"lfmpwd"
         local cv = cookie and cookie:value()
         if cv then
            return bd.aesdecode(password) == ba.aesdecode(getKey(), cv)
         end
      end
      return false
   end
   checkpasswd = _checkpasswd
end

local function showlogin(_ENV)
   title="Login"
   response:include"/rtl/.bdlua/maildir/pwdform.lsp"
   return true
end

local function showNoPasswd(_ENV)
   response:include"/rtl/.bdlua/maildir/nopwd.lsp"
   return true
end



local function setPassword(_ENV,conn,key)
   _ENV.conn=conn
   _ENV.key=key
   response:include"/rtl/.bdlua/maildir/setpwd.lsp"
   return true
end

function logDownload(ipaddr,mlkey,filekey,err)
   if mlkey and filekey then
      local function log()
         local env,conn=bd.openDb"bd"
         local mjk=sqlutil.find(conn,fmt("%s%u","mailjobkey FROM mailjobfile WHERE key=",filekey))
         if mjk then
            local sql=fmt("%s (%s,%u,%u,%s,%u,%s)",
                          "INSERT INTO maildownload (mailjobkey,filekey,mlkey,ipaddr,date,err) VALUES",
                          mjk,filekey,mlkey,quote(ipaddr),os.time(),err and quote(err) or "NULL")
            mailutil.sqlexec(conn,sql)
         end
         bd.closeDb(env,conn)
      end
      ba.thread.run(log)
   end
end


local function asyncResp(s,fp,size,ipaddr,mailcontactkey,mailjobfilekey)
   local ok,e,d=true,nil,nil
   while ok and size > 0 do
      d,e=fp:read(1400)
      if not d then break end
      ok,e=s:write(d)
      size = size - #d
   end
   e = size~=0 and (e or "failed")
   logDownload(ipaddr,mailcontactkey,mailjobfilekey,e)
   fp:close()
   if not e then s:read(10000) end -- Wait for all data to be sent
   s:close()
end


local function batchJobFailed(_ENV,cmd,emsg)
   if type(emsg) ~= "string" then emsg = tostring(emsg) or "?" end
   emsg=fmt('Batch job "%s" failed: %s',cmd,emsg)
   mailutil.seterr(emsg)
   title="Batch Job Failed"
   response:include"/rtl/.EmptyHeader.lsp"
   response:write('<h1>',title,'</h1>')
   response:write('<p>',emsg,'</p>')
   response:include"/rtl/.EmptyFooter.lsp"
   return true
end


local function download(_ENV,batchkey,riop,name,mailcontactkey,mailjobfilekey)
   local fn=riop..name
   local st=rio:stat(fn)
   if request:method() ~= "POST" and not request:data"password" then
      response:include"/rtl/.bdlua/maildir/download.lsp"
      return true
   end


   if st then
      if batchkey then
         local email,id=sqlutil.find(conn,fmt("email,id FROM mailcontact WHERE key=%u",mailcontactkey))
         local cmd=sqlutil.find(conn,fmt("cmd FROM mailbatchjob WHERE key=%u",batchkey))
         if not email or not cmd then return false end
         cmd=fmt("%s %s %s%s",cmd,rio:realpath(fn), email, id and fmt(' "%s"',id) or '')
         local rsp,err=ba.exec(cmd)
         if not rsp then return batchJobFailed(_ENV,cmd, err or "No response from batch script") end
         if #rsp > 0 then
            local nfn=bd.trim(rsp)
            -- Dos2unix (for windows)
            if nfn:find"^(%w)(:)" then
               nfn=nfn:gsub("^(%w)(:)", "%1",1)
               nfn=nfn:gsub("\\", "/")
            end
            st=rio:stat(nfn)
            if not st then return batchJobFailed(_ENV,cmd, fmt("%s: %s","Batch script returned error message",rsp)) end
            fn=nfn
            name=fn:match"([^/]+)$"
         end
      end
      local fp=rio:open(fn)
      if fp then
         local ipaddr=request:peername()
         local r=response
         r:reset()
         r:setheader("Content-type","application/octet-stream")
         r:setheader("Content-Disposition",fmt("attachment; filename=\"%s\"",name))
         r:setheader("Cache-Control", "no-store, no-cache, no-transform, must-revalidate, private")
         r:setheader("Pragma","private")
         r:setheader("Expires","0")
         r:setcontentlength(st.size)
         if st.size > 80000 then -- Use asynch send
            r:downgrade()
            r:flush()
            local s=ba.socket.req2sock(r)
            s:event(asyncResp,"s",fp,st.size,ipaddr,mailcontactkey,mailjobfilekey)
         else
            r:send(fp:read"*a")
            fp:close()
            logDownload(ipaddr,mailcontactkey,mailjobfilekey)
         end
         return true
      end
   end
   return false
end


local function getAction(_ENV,mailjobfilekey,mailjobhash,mailcontactkey,password)
   local action
   local mailjobkey,name=sqlutil.find(
      conn,fmt("%s%u","mailjobkey,name FROM mailjobfile WHERE key=",mailjobfilekey))
   if mailjobkey then
      local t={}
      sqlutil.findt(conn, "hash,riop,secure,password,expires,batchkey FROM mailjob WHERE key="..mailjobkey,t)
      if mailjobhash == t.hash then
         if t.secure == "1" and not request:issecure() then
            action = redir2securl
         elseif t.expires and t.expires ~= "0" and tonumber(t.expires) < os.time() then
            action = expiredinfo
         elseif mailcontactkey and t.password == "1" and not checkpasswd(_ENV,password) then
            if password then
               action=showlogin
            else
               action=showNoPasswd
            end
         else
            action=function(_ENV) return download(_ENV,t.batchkey,t.riop,name,mailcontactkey,mailjobfilekey) end
         end
      end
   end
   return action
end


local function emailDirFunc(_ENV,relpath)
   local action
   if relpath:sub(-1,-1) == "/" then
      relpath=relpath:sub(1,-2)
   end
   local env,conn=bd.openDb"bd"
   _ENV.conn=conn
   local rl=#relpath
   if rl > 20 then
      local hl = relpath:byte(1) - 45
      if hl > 19 and hl < 35 then
         local contacthash
         local mailjobhash
         if hl+1 == rl then
            contacthash=relpath:sub(2,-1)
         else
            contacthash=relpath:sub(2,hl+1)
            mailjobhash=relpath:sub(hl+2,hl+3)
         end
         local mailcontactkey,password=sqlutil.find(
               conn, "key,password FROM mailcontact WHERE hash="..quote(contacthash))
         if mailcontactkey then
            if mailjobhash then
               local mailjobfilekey=relpath:sub(hl+4,-1)
               mailjobfilekey=tonumber(mailjobfilekey,16)
               if mailjobfilekey then
                  action=getAction(_ENV,mailjobfilekey,mailjobhash,tonumber(mailcontactkey),password)
               end
            else
               local a=request:data"a"
               if a == "pwd" and not password then
                  if mailutil.settings(conn,"confirmpwdsec") == "1" and not request:issecure() then
                     redir2securl(_ENV)
                  end
                  action=function(_ENV) return setPassword(_ENV,conn,mailcontactkey) end
               else
                  _ENV.mailcontactkey=mailcontactkey
                  _ENV.conn=conn
                  response:include"/rtl/.bdlua/maildir/settings.lsp"
                  return true
               end
            end
         end
      end
   elseif rl > 8 then
      local mailjobhash=relpath:sub(1,8)
      local mailjobfilekey=tonumber(relpath:sub(9,-1),16)
      if mailjobfilekey then
         action=getAction(_ENV,mailjobfilekey,mailjobhash)
      end
   end
   local ret=action and action(_ENV,conn) or false
   bd.closeDb(env,conn)
   return ret
end

return emailDirFunc
