<?lsp

local tinsert=table.insert
local fmt=string.format
local testOK, testErrMsg


function loadSmtpFromCfg()
   local cfg=bd.cfg.load()
   local smtp=cfg.v.smtp
   if smtp then
      smtp.password=bd.aesdecode(smtp.password)
   end
   return smtp
end

local function smtpOK(smtp)
   if smtp.email and
      smtp.server and
      smtp.port and
      smtp.uname and
      smtp.password and
      smtp.accounttype and
      smtp.consec then
      return true
   end
   return nil, "Inv smtp settings"
end

function saveSmtpToCfg(smtp)
   if smtpOK(smtp) then
      local cfg=bd.cfg.load()
      local pwd=smtp.password
      smtp.password=bd.aesencode(smtp.password)
      cfg.v.smtp=smtp
      bd.cfg.save(cfg)
      smtp.password=pwd
   end
end


function loadSmtpFromBdDb(env,conn)
   local closeDb
   local email,server,port,useauth,uname,password,accounttype,consec
   local sqlutil=require"sqlutil"
   if not env then
      env,conn = bd.openDb"bd"
      closeDb=true
   end
   email,server,port,useauth,uname,password,accounttype,consec=
      sqlutil.find(conn,"email,server,port,useauth,uname,password,accounttype,consec FROM mailsmtp")
   if closeDb then bd.closeDb(env,conn) end
   return #email > 0 and {
      email=email,server=server,port=port,useauth=useauth=="1",uname=uname,
      password=password and bd.aesdecode(password) or "",accounttype=accounttype,consec=consec
   }
end

function saveSmtpToBdDb(smtp,env,conn)
   local ok,e=smtpOK(smtp)
   if ok then
      local closeDb
      local q=luasql.quotestr
      local sql=fmt("UPDATE mailsmtp SET email=%s,server=%s,port=%u,useauth=%u,uname=%s,password=%s,accounttype=%s,consec=%s",
                    q(smtp.email),q(smtp.server),smtp.port,smtp.useauth and 1 or 0,
                    q(smtp.uname),q(bd.aesencode(smtp.password)),q(smtp.accounttype),q(smtp.consec))
      if not env then
         env,conn = bd.openDb"bd"
         closeDb=true
      end
      ok,e=require"mailutil".sqlexec(conn,sql)
      if closeDb then bd.closeDb(env,conn) end
   end
   return ok,e
end






local function getChecked(attr, val)
   if attr == val then
      return ' checked="checked"'
   end
   return ""
end

local function emitCheckbox(name, isChecked)
   response:write('<input type="checkbox" name="',name,'" id="',name,'"',
                  isChecked and " checked" or "",'/>')
end

local function emitRadio(group, val, attr)
   response:write('<input type="radio" name="',group,'" value="',
                  val ,'"', getChecked(attr,val),'/>')
end

local function emitRadios(group,tab,vals,texts)
   if texts then
      for k,v in ipairs(vals) do
         emitRadio(group, v, tab[group])
         response:write("&nbsp;",texts[k])
      end
   else
      for _,v in ipairs(vals) do
         emitRadio(group, v, tab[group])
      end
   end
end


function createSmtp()
   return {
      email="",
      server="",
      port=25,
      uname="",
      password="",
      accounttype="generic",
      consec="none",
   }
end

----------------------------------------
function emitSMTP()

local data=bd.trim(request:data())
if request:method()=="POST" and (data.smtpbut or data.testmailbut) then
   smtp=data
   smtp.ok = false
   if #smtp.email > 0 and #smtp.server > 0 then
      if smtp.useauth then
         smtp.ok = #smtp.uname > 0 and #smtp.password > 0
      else
         smtp.ok=true
      end
   end
   smtp.smtpbut=nil
   local test = smtp.testmailbut
   smtp.testmailbut=nil
   smtp.port=tonumber(smtp.port)
   if not smtp.port then smtp.port=25 end
   local pwd=smtp.password
   savesmtp(smtp)
   smtp.password=pwd
   if test then
      local m={
         subject="FuguHub SMTP test",
         body="Email sent from FuguHub."
      }
      testOK,testErrMsg=sendTestMail(m,smtp)
   end
else
   smtp.ok = smtp.email and #smtp.email > 0 and smtp.server and #smtp.server > 0
end
if testOK or testErrMsg then
   if testOK then
      response:write'<fieldset><legend><b>SMTP Settings </b></legend><h1>Test Successful</h1>'
      response:write('<p>An Email was sent to ',smtp.email,'. Check your mailbox.</p>')
   else
      response:write'<h1>Test Failed</h1>'
      response:write('<p>The Email server sent the following error message: ',testErrMsg,'</p>')
   end
   response:write('<p><a href="',request:uri(),'">Continue</a></p></fieldset>')
else
?>
<form method="post" autocomplete="off">
<fieldset>
  <legend><b> SMTP Settings </b></legend>
  <?lsp=helpdata or ""?>
  <table>
    <tr><td>
      <div id="accounttypediv" style="display:none">
        <fieldset>
          <legend><b> Account Type </b></legend>
          <?lsp emitRadios("accounttype", smtp, {"gmail", "hotmail", "generic"},
                   {"Gmail&nbsp;&nbsp;", "Hotmail&nbsp;&nbsp;", "Generic"})
          ?>
        </fieldset>
      </div>
    </td></tr>
    <tr><td>
    <fieldset id="settings">
      <legend><b> Settings </b></legend>
      <table>
        <tr><td><b>Email Addr:</b></td><td><input class="wide" type="text" name="email" value="<?lsp=smtp.email?>" placeholder="firstname lastname <name@domain.xx>"/></td></tr>
        <tr><td><b>Server&nbsp;Name:</b></td><td><input class="wide" type="text" name="server" value="<?lsp=smtp.server?>" placeholder="Enter the SMTP server name"/></td></tr>
        <tr><td><b>Port&nbsp;Number:</b></td><td><input class="wide" type="text" name="port" value="<?lsp=smtp.port?>"/></td></tr>
      </table>
    </fieldset>
  </td></tr>
  <tr><td>
    <fieldset>
      <legend><b> Security and Authentication </b></legend>
      <?lsp emitCheckbox("useauth",smtp.useauth) ?> Use Name and Password
      <table id="useauthtab" style="margin-left:20px">
        <tr><td><b>User Name:</b></td><td><input class="wide" type="text" name="uname" value="<?lsp=smtp.uname?>" placeholder="Enter your username if required by SMTP server"/></td></tr>
        <tr><td><b>Password:</b></td><td><input class="wide" type="password" name="password" value="<?lsp=smtp.password?>" placeholder="Enter your password if required by SMTP server"/></td></tr>
        <tr><td colspan="2">
        <fieldset id="consecfield">
          <legend><b> Connection Security </b></legend>
          <?lsp emitRadios("consec", smtp, {"none", "tls", "starttls"},
                   {"None", "SSL/TLS", "STARTTLS"})
          ?>
        </fieldset>
        </td></tr>
      </table>
    </fieldset>
  </td></tr>
  </table>
  <?lsp
    if smtp.ok and extCheckbox then
      for _,k in ipairs(extCheckbox) do
         emitCheckbox(k,smtp[k])
         response:write" Enable FuguHub User Password Recovery<br/><br/>"
      end
    end
  ?>
  <input type="submit" name="smtpbut" value="Save" />
  <?lsp if smtp.ok then ?>
  <input type="submit" id="testmailbut" name="testmailbut" value="Save and Test" />
  <?lsp end
       end
  ?>
</fieldset>
</form>
<script type="text/javascript" src="/rtl/showwait.js"></script> 
<script type="text/javascript">$("#testmailbut").click(function() {showwait();});</script> 
<?lsp end -- function emitSMTP() ?>
