<?lsp

-- Web-Services for adding and modifying blog entries.

local jsonrpc=require"jsonrpc"

local fmt=string.format

local function title2url(s)
   return s:gsub("[^%a%d%s]", ""):gsub("^%s*(.-)%s*$", "%1"):gsub("%s+", "-")
end

local function listCategories()
   local cur, e = conn:execute("SELECT rowid,category FROM blogcategories ORDER BY category DESC")
   if not cur then error(e) end
   local categories={}
   local rowid,name = cur:fetch()
   while name do
      table.insert(categories, {rowid=rowid,name=name})
      rowid,name = cur:fetch()
   end
   cur:close()
   return categories
end

local function addCategory(category)
local qfind = fmt([[SELECT rowid,category FROM blogcategories WHERE category='%s']], category)
   local na,e
   category=category:gsub("'","''")

   local cur, e = conn:execute(qfind)
   if cur then
      local rowid,name = cur:fetch()
      cur:close()
      if rowid then
         error(fmt('\nERROR: Category "%s" already in database.', category))
      end
   end

   na,e=conn:execute(fmt(
    [[INSERT INTO blogcategories (category) values ('%s')]],
    category))
   if e then error(e) end
   app.categoriesUpdated()
   local cur, e = conn:execute(qfind)
   if not cur then error(e) end
   local rowid,name = cur:fetch()
   cur:close()

   return  {rowid=rowid,name=name}
end


local function removeCategory(rowid)
   local na,e
   na,e=conn:execute(fmt(
      [[DELETE FROM blogcategories WHERE rowid = %s]], rowid),1000)
   if e then error(e) end
   app.categoriesUpdated()
   return  "ok"
end

local function categoryInfo(rowid)
   local cur,e
   cur,e=conn:execute(fmt(
      [[SELECT DISTINCT category, description, ttl, maxlen, image FROM blogcategories WHERE rowid = %s]], rowid))
   if e then error(e) end
   local category, description, ttl, maxlen, image = cur:fetch()
   cur:close()
   if not category then error("Not found") end
   local rss = app.rss
   return  {
      category=category,
      description = description and description or rss.description,
      ttl = ttl and ttl or rss.ttl,
      maxlen = maxlen and maxlen or rss.maxlen,
      image = image and image or rss.image 
   }
end

local function fixImageUrls(data)
   local host = request:header"host"
   if host then
      -- In the editor, some browsers create an absolute link URL for images.
      host = fmt("(https?://%s)",host:gsub("([%[%^%$%(%)%%%.%[%]%*%+%-%?])",
                                           function(x) return "%"..x end))
      data = data:gsub(host, function() return "" end)
   end
   return data
end



local function updateBlog(rowid, category, subject, name, data)
   local na,e
   data=fixImageUrls(data)
   na,e=conn:execute(fmt(
      [[UPDATE blog SET category='%s',subject='%s',url='%s',name='%s',data='%s' WHERE ROWID=%s]],
      category:gsub("'","''"), 
      subject:gsub('([<>])',""):gsub("'","''"), 
      title2url(subject):gsub("'","''"), 
      name:gsub("'","''"), 
      data:gsub("'","''"),
      rowid))
   if e then error(e) end
   app.emptyPageCache()
   return  "ok"
end

local function addBlog(category, subject, name, data)
   local na,e
   data=fixImageUrls(data)
   na,e=conn:execute(fmt(
     [[INSERT INTO blog (date,category,subject,url,name,data) values ('%u','%s','%s','%s','%s','%s')]],
     os.time(),
     category:gsub("'","''"), 
     subject:gsub('([<>])',""):gsub("'","''"),
     title2url(subject):gsub("'","''"), 
     name:gsub("'","''"),
     data:gsub("'","''")))
   if e then error(e) end
   app.updateArchList()
   app.emptyPageCache()
   return  "ok"
end


local function loadBlog(rowId)
   local cur,e=conn:execute(fmt(
      [[SELECT DISTINCT data FROM blog WHERE ROWID=%s]],
      rowId))
   if e then error(e) end
   local data=""
   if cur then
      data = cur:fetch()
      cur:close()
   end
   return data
end



local function getInfo(rowid)
   local cur, e = conn:execute(fmt("SELECT category,name FROM blog WHERE ROWID=%s", rowid))
   if not cur then error(e) end
   local category,name = cur:fetch()
   cur:close()
   return {categories=listCategories(),category=category,name=name}
end


local service = jsonrpc.new("Blog",
                               {
                                  blog={
                                     update=updateBlog,
                                     add=addBlog,
                                     load=loadBlog,
                                  },
                                  category={
                                     remove=removeCategory,
                                     add=addCategory,
                                     list=listCategories,
                                     info=categoryInfo,
                                  },
                                  info={
                                     get=getInfo,
                                  }
                               })
env,conn = app.openDb()
response:reset()
collectgarbage()
collectgarbage()
service:execute(request, response)
app.closeDb(env,conn)
request:session().cmsIsAdmin=true
?>
