<?lsp

io=app.docsIo

-- Fetch and validate current path
local d = data or request:data(); -- Get URL data (data may be set by caller)
local rpath = d.path or "/"; -- Set path to top if not found

--Create aliases so we do not have to type so much
local fmt = string.format
local function prtf(...) response:write(fmt(...)) end
local function wrt(...) response:write(...) end

local function getImgMimeType(name)
   local ext = string.match(name, "%.(%w-)$")
   if ext then
      ext=string.lower(ext)
      local mime={
         jpg="image/jpeg",
         jpeg="image/jpeg",
         gif="image/gif",
         png="image/png"
      }
      mime = mime[ext]
      if mime then
         return mime
      end
   end
   return nil
end


-- returns rpath + path and makes sure we do not get path1//path2
local function mkPath(rpath,path)
   return string.gsub(rpath .. "/" .. path, "//" ,"/")
end


-- Make string URL friendly.
-- From the Lua book: http://www.lua.org/pil/20.3.html
-- We made a small correction to how this function works.
local function escape (s)
   s = string.gsub(s, "([^%w])",
                   function (c)
                      return string.format("%%%02X", string.byte(c))
                   end)
   return s
end

-- Creates a navigation URL such as ./?path=thepath, where thepath is
-- the argument to this function. This function also preserves all
-- other URL encoded parameters received from the client.  Notice how
-- we make this function part of the call object.  We do this to make
-- it possible for the calling LSP page to use this function.
function encodeURL(path, normURL)
   d.path = path; -- Set the path attribute. d is request:data()
   local s=""; -- The URL encoded data
   for k,v in pairs(d) do
      s = s .. "&" .. escape(k) .. "=" .. escape(v)
   end
   return "?"..string.sub(s, 2)     -- remove first '&' and add URL.
end

-- This function emits the navigation links to the response.
-- The navigation links make it easy to traverse the parent directories.
local function emitNavigationLinks()
   local curDir="top"; -- defaults to top directory.
   print("<div id='menu'><span id='navlink'>")
   if(rpath == "/") then -- If top URL
      wrt("/");
   else -- Not a top URL, create the links to traverse the file manager.
      prtf("<a href='%s' rp='%s'>top</a>",
           encodeURL("/"), io:realpath(""));
      local uri="/"
      local path=""
      local last=""
      string.gsub(rpath,"/([^/]+)", -- extract all '/path' elements
                  function (h)
                     path = fmt("%s/%s",path,h)
                     uri = mkPath(uri,h); -- uri = uri + h
                     prtf("%s/",last); -- emit link from last call
                     last = fmt("<a href='%s' rp='%s'>%s</a>",
                                encodeURL(uri), io:realpath(path), h);
                     curDir = h; -- Last 'path' will be current directory.
                  end
               );
   end
   prtf("</span><span id='curdir' rp='%s'>Directory '%s'</span></div>",
        io:realpath(rpath),curDir);
end


--- Create the Web-File-Manager
function emitWebFileManager()
   if gd then
?>
<form method="post" action="photo/cfgupload.lsp">
<input type="hidden" name="doneUplUrl" value="<?lsp=request:uri()?>"/>
<input style="float:right;margin:5px" type="submit" value="Upload Images" />
</form>
<?lsp
   end
   emitNavigationLinks(); -- Start by creating the navigation links.

   -- Emit HTML table with links to resources in current directory.
   -- Current directory is 'rpath', which was extracted from the URL
   -- encoded data above.
   wrt("<table>",
       "<thead><tr><th>Name</th><th>Size</th><th>Date</th></tr></thead><tbody>");
   -- Use the application OS object to give us a file iterator for current dir
   for v in io:files(rpath) do
      local fn = mkPath(rpath, v);  -- fn = rpath + v
      local sb = io:stat(fn);  -- Should not fail as rpath + v must be valid.
      if sb and ((sb.isdir and (v~=".LOCK" and v~=".DAV")) or getImgMimeType(fn)) then
         wrt("<tr><td>"); -- Emit HTML table row and create links to resource
         if sb.isdir and  string.sub(v,1,1) ~= '.' then
            prtf("<a href='ImageSelector.lsp%s/'>%s</a><br/>", encodeURL(fn), v);
         else
            prtf("<a class='imgt' href='%s' name='%s'>%s</a><br/>",
                 escape(fn),escape(v),v);
         end
         prtf("</td><td>%s</td><td>%s</td></tr>\n",
              sb.isdir and
              "<img class='dir' src='/rtl/bindows/html/images/folder.png' alt='DIR'/>"
              or sb.size.."",
              os.date("%c",sb.mtime))
      end
   end
   wrt("</tbody></table>");
end

do
   local st = io:stat(rpath)
   if rpath == "" or not st then
      response:senderror(404)
      return
   end
   if not st.isdir then
      response:reset()
      local mime=getImgMimeType(rpath)
      if mime then
         local f = io:open(rpath)
         if f then
            response:setcontentlength(st.size)
            response:setcontenttype(mime)
            response:send(f:read(st.size))
            f:close()
            return
         end
      end
      response:senderror(404)
      return
   end
end
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<link href='/rtl/wfm/style.css' rel='stylesheet' type='text/css' />
<style type="text/css">
#top {
padding: 1em;
}

#preview {
  position: absolute;
  display:none;
  z-index:1;
}
#PreviewImg {
   margin:0;
   border: outset;
   display:none;
   background:white;
}

#PreviewTxt {
   margin:0;
   border: outset;
   background:white;
   padding:1em;
}

#rp {
   border:1px solid black;
   background: white;
   margin: 0 1em 0 1em;
   padding:.5em;
   position: absolute;
   z-index:1;
   display:none;
}


</style>
<script type='text/javascript' src='/rtl/jquery.js'></script>
<script type='text/javascript' src='ImageSelector.js'></script>
</head>
<body>
<div id='rp'><b>Real Path:</b> <span id='rptxt'></span></div>
<div id="preview">
<p id="PreviewTxt"></p>
<img id="PreviewImg" alt="PreviewImg"/>
</div>
<?lsp emitWebFileManager(); ?>
</body>
</html>
