
/*
The DomElemList class manages a list of clickable elements.
divId the id attribute name of the DIV containing the list/
delButtonId the id attribute name of the delete button
bgColor the background color used in the DIV element.

 */
function DomElemList(divId, delButtonId, bgColor)
{
   this.$bgColor=bgColor;
   this.$contDiv=document.getElementById(divId);
   var delBut=document.getElementById(delButtonId);
   if(this.$contDiv)
   {
      if(delBut)
      {
         delBut.$list=this;
         delBut.onclick=function() {this.$list.$delButOnClick();};
      }
      this.$selectedElem=null;
      return;
   }
   throw "Unknown ID";
};


/*
  print (alert) user if connection failed.
*/
function printConnectError(exception)
{
      var em;
      if(exception.toTraceString)
         em=exception.toTraceString();
      else
         em = exception.message;

      var eMsg = "Cannot connect to server.\nMake sure you are authenticated.\nClose this popup window and press refresh in the parent window.\n\nError code:\n";
      alert(eMsg+em);
}


/*
Install an object with an onClick and onDelete method. The methods are
called when the user clicks an element or clicks the delete button on
a selected element.
 */
DomElemList.prototype.setCallbackObj=function(obj)
{
   if(typeof obj.onClick != 'function' ||
      typeof obj.onDelete != 'function')
   {
      throw "Invalid CallbackObj";
   }
   this.$cbObj=obj;
};


/* Add an element to the list
 */
DomElemList.prototype.add=function(name)
{
   var d = document.createElement('div');
   d.appendChild(document.createTextNode(name));
   d.onclick=function() {this.$list.$elemOnClick(this);};
   d.$list=this;
   d.$name=name;
   var s=d.style;
   s.width="100%";

   for(var dIter = this.$contDiv.firstChild; dIter; dIter = dIter.nextSibling)
   {
      if(dIter.$name > name)
      {
         this.$contDiv.insertBefore(d, dIter);
         return;
      }
   }
   this.$contDiv.appendChild(d);
};

/* remove an element from the list
 */
DomElemList.prototype.remove=function(name)
{
   for(var d = this.$contDiv.firstChild; d; d = d.nextSibling)
   {
      if(d.$name == name) this.$contDiv.removeChild(d);
   }
};

/* Removes all elements in the list.
 */
DomElemList.prototype.removeAll=function()
{
   for(var d = this.$contDiv.firstChild; d;)
   {
      var n = d;
      d = d.nextSibling;
      if(n.$name)
      {
         this.$contDiv.removeChild(n);
      }
   }
};


/* Clears the selected element i.e. restores the background color.
 */
DomElemList.prototype.clearSelected=function()
{
   for(var d = this.$contDiv.firstChild; d; d = d.nextSibling)
   {
      if(d.style)
         d.style.backgroundColor=this.$bgColor;
   }
   this.$selectedElem=null;
}

/* Activated when user clicks the delete button
 */
DomElemList.prototype.$delButOnClick=function()
{
   if(this.$selectedElem)
   {
      this.$cbObj.onDelete(this.$selectedElem.$name);
      this.$contDiv.removeChild(this.$selectedElem);
      this.$selectedElem=null;
   }
};


/* Activated when user clicks on an element in the list
 */
DomElemList.prototype.$elemOnClick=function(divElem)
{
   for(var d = this.$contDiv.firstChild; d; d = d.nextSibling)
   {
      if(d.style)
         d.style.backgroundColor=this.$bgColor;
   }
   divElem.style.backgroundColor="gray";
   this.$selectedElem = divElem;
   this.$cbObj.onClick(this.$selectedElem.$name);
};


/* This function converts an array of strings to a string suitable for
 * the HTML text elements.
*/
function array2FieldNames(a)
{
   var names="";
   for(var i = 0; i < a.length; i++)
      names += a[i]+'\n';
   return names;
};


/* This function is the opposite of array2FieldNames
 */
function fieldNames2Array(names)
{
   var a = names.split(/[\s,]+/);
   var retVal=[];
   for(var i = 0; a[i] ; i++)
   {
      if(a[i].length != 0)
         retVal.push(a[i]);
   }
   return retVal;
};
